@@ -11,8 +11,9 @@ from jsonschema import ValidationError, validate as jsvalidator
 from yaspin import yaspin
 
 from build_magic import actions, output, runner
-from build_magic.exc import ExecutionError, NoJobs, SetupError, TeardownError
+from build_magic.exc import ExecutionError, NoJobs, OSEnvironmentMismatch, SetupError, TeardownError
 from build_magic.macro import MacroFactory
+from build_magic.reference import VARIABLE_PATTERN
 from build_magic.reference import Actions, Directive, ExitCode, OutputMethod, OutputTypes, Runners
 from build_magic.reference import BindDirectory, HostWorkingDirectory, KeyPassword, KeyPath, KeyType
 
@@ -89,31 +90,46 @@ def config_parser(config):
         data = data.get('stage', {})
         stage = dict()
         stage['name'] = data.get('name', '')
+        stage['description'] = data.get('description', '')
         stage['runner_type'] = data.get('runner', Runners.LOCAL.value)
         stage['environment'] = data.get('environment', '')
         stage['continue'] = data.get('continue on fail', False)
         stage['wd'] = data.get('working directory', '.')
         stage['copy'] = data.get('copy from directory', '')
         stage['artifacts'] = data.get('artifacts', [])
+        stage['dotenv'] = data.get('dotenv', '')
 
         # Set the action.
         stage['action'] = data.get('action', Actions.DEFAULT.value)
 
-        # Set the command and directives.
+        # Set the command, directives, and labels.
         macros = data.get('commands', [])
         commands = []
         directives = []
+        labels = []
         for macro in macros:
+            if 'label' in macro.keys() and len(macro.keys()) > 1:
+                labels.append(macro['label'])
+            else:
+                labels.append('')
             directives.append(list(macro.keys())[0])
             commands.append(list(macro.values())[0])
         stage['commands'] = commands
         stage['directives'] = directives
+        stage['labels'] = labels
 
         # Set the parameters.
-        parameters = data.get('parameters', [])
-        stage['parameters'] = [tuple(param.items())[0] for param in parameters]
+        parameters = data.get('parameters', {})
+        stage['parameters'] = list(parameters.items())
         stages.append(stage)
 
+        # Set the environment variables.
+        envs_raw = data.get('environment variables', {})
+        envs = {}
+        for key, value in envs_raw.items():
+            envs.update({key: value})
+        stage['environment variables'] = envs
+
     return stages
 
 
@@ -125,17 +141,21 @@ def parse_variables(config, variables):
     :rtype: dict
     :return: The resulting config after variable substitution.
     """
-    pattern = r'({{\s?\w+\s?}})'
     matched = False
     config_string = json.dumps(config)
     if variables:
-        if re.search(pattern, json.dumps(config)):
-            matches = re.findall(pattern, json.dumps(config))
+        if re.search(VARIABLE_PATTERN, json.dumps(config)):
+            matches = re.findall(VARIABLE_PATTERN, json.dumps(config))
             if matches:
                 for key, value in variables.items():
                     for match in matches:
                         if key in match:
-                            config_string = re.sub(r'{{\s?' + key + r'\s?}}', value, config_string)
+                            # Convert value that might be Windows path.
+                            if '\\' in value:
+                                value = value.encode('unicode-escape').decode('utf-8')
+                                value = value.replace('\\', '/')
+                            pattern = re.compile(r'{{\s?' + key + r'\s?}}')
+                            config_string = re.sub(pattern, value, config_string)
                             matched = True
                             break
             if not matched:
@@ -190,7 +210,8 @@ class Engine:
         for stage in self._stages:
             spinner = yaspin()
             # Run the stage.
-            _output.log(mode.STAGE_START, stage.sequence, stage.name)
+            _output.log(mode.STAGE_START, stage.sequence, stage.name, stage.description)
+            _output.log(mode.WORKING_DIRECTORY, stage.command_runner.working_directory)
             # Launch the process spinner
             _output.log(mode.PROCESS_SPINNER, spinner, process_active=True)
             stage.setup()
@@ -204,10 +225,10 @@ class Engine:
                 raise err
 
             if exit_code > status_code:
-                status_code = exit_code
+                if not (exit_code == ExitCode.SKIPPED and stage.sequence < len(self._stages)):
+                    status_code = exit_code
             _output.log(mode.STAGE_END, stage.sequence, exit_code, stage.name)
 
-        # TODO: This is the wrong status code - fix it.
         _output.log(mode.JOB_END)
         return status_code
 
@@ -216,18 +237,19 @@ class StageFactory:
     """Validates and generates Stage objects."""
 
     @classmethod
-    def _build_macros(cls, commands):
+    def _build_macros(cls, commands, labels):
         """Build Macro objects from provided commands.
 
         :param list[str] commands: The commands to use for building the Macro objects.
+        :param list[str] labels: The command descriptions to display.
         :rtype: list[Macro]
         :return: A list of Macro objects.
         """
-        factory = MacroFactory(commands)
+        factory = MacroFactory(commands, labels=labels)
         return factory.generate()
 
     @classmethod
-    def _build_command_runner(cls, runner_type, environment, parameters, copy, wd, artifacts):
+    def _build_command_runner(cls, runner_type, environment, parameters, copy, wd, artifacts, envs):
         """Build a CommandRunner object.
 
         :param str runner_type: The command runner to use for executing commands.
@@ -236,6 +258,7 @@ class StageFactory:
         :param str wd: The working directory to use for executing commands.
         :param list[str] artifacts: The Stage artifacts to work on.
         :param dict parameters: The user-supplied parameters to pass to the command runner.
+        :param dict envs: The environment variables to set.
         :rtype: CommandRunner
         :return: The CommandRunner object to use for executing the Stage's commands.
         """
@@ -252,7 +275,14 @@ class StageFactory:
             raise NotADirectoryError(f'Path {copy} does not exist.')
 
         command_runner = getattr(runner, runner_type.capitalize())
-        return command_runner(environment, working_dir=wd, copy_dir=copy, artifacts=artifacts, parameters=parameters)
+        return command_runner(
+            environment,
+            working_dir=wd,
+            copy_dir=copy,
+            artifacts=artifacts,
+            parameters=parameters,
+            envs=envs,
+        )
 
     @classmethod
     def _build_parameters(cls, parameters):
@@ -288,6 +318,10 @@ class StageFactory:
             wd,
             name=None,
             parameters=None,
+            envs=None,
+            labels=None,
+            description=None,
+            skip=False,
     ):
         """Validates inputs and generates a new Stage object.
 
@@ -302,6 +336,10 @@ class StageFactory:
         :param str wd: The working directory to use for executing commands.
         :param str|None name: The stage name if provided.
         :param list[tuple]|None parameters: The optional parameters passed by the user.
+        :param dict|None envs: A dictionary of environment variables to use.
+        :param dict|None labels: Command descriptions.
+        :param str|None description: A description of the stage.
+        :param bool skip: If True, indicates the stage should be skipped.
         :rtype: Stage
         :return: The generated Stage object.
         """
@@ -316,9 +354,15 @@ class StageFactory:
                     'Directive must be one of {}'.format(', '.join(Directive.available()))
                 )
 
+        if labels is None:
+            labels = [''] * len(commands)
+
         if len(commands) != (len(directives)):
             raise ValueError('Length of commands unequal to length of directives.')
 
+        if len(labels) != len(commands):
+            raise ValueError('Length of commands unequal to length of labels.')
+
         if action not in Actions.available():
             raise ValueError('Action must be one of {}.'.format(', '.join(Actions.available())))
 
@@ -328,8 +372,11 @@ class StageFactory:
         if not artifacts:
             artifacts = []
 
+        if not envs:
+            envs = {}
+
         # Build the macros.
-        macros = cls._build_macros(commands=commands)
+        macros = cls._build_macros(commands=commands, labels=labels)
         if not macros:
             raise ValueError('There are no commands to execute.')
 
@@ -341,9 +388,10 @@ class StageFactory:
             wd=wd,
             artifacts=artifacts,
             parameters=params,
+            envs=envs,
         )
 
-        return Stage(cmd_runner, macros, directives, sequence, action, name)
+        return Stage(cmd_runner, macros, directives, sequence, action, name, description, skip)
 
 
 class Stage:
@@ -359,9 +407,11 @@ class Stage:
         '_results',
         '_sequence',
         '_name',
+        '_description',
+        '_skip',
     ]
 
-    def __init__(self, cmd_runner, macros, directives, sequence, action, name=None):
+    def __init__(self, cmd_runner, macros, directives, sequence, action, name='', description='', skip=False):
         """Instantiates a new Stage object.
 
         Note: Stage objects should not be constructed directly and should instead be created by a StageFactory.
@@ -371,7 +421,9 @@ class Stage:
         :param list[str] directives: The command directives.
         :param int|str sequence: The execution order of the macros.
         :param str action: The Action to use.
-        :param str|None name: The stage name if provided.
+        :param str name: The stage name if provided.
+        :param str description: The description of the stage if provided.
+        :param bool skip: If True, the stage should be skipped (not run by the Engine).
         """
         try:
             self._action = getattr(actions, action.capitalize())
@@ -386,6 +438,8 @@ class Stage:
         self._result = 0
         self._is_setup = False
         self._name = name
+        self._description = description
+        self._skip = skip
 
     @property
     def sequence(self):
@@ -400,7 +454,22 @@ class Stage:
     @property
     def name(self):
         """The stage name."""
-        return self._name
+        return self._name or ''
+
+    @property
+    def description(self):
+        """The stage description."""
+        return self._description or ''
+
+    @property
+    def skip(self):
+        """True if the stage should be skipped from execution."""
+        return self._skip
+
+    @property
+    def command_runner(self):
+        """The command runner associated with the stage."""
+        return self._command_runner
 
     def _get_action_function(self, method):
         """Fetches the mapped action function for the provided command runner method.
@@ -434,6 +503,16 @@ class Stage:
         :param yaspin spinner: The yaspin spinner object to use.
         :return: The highest Stage result.
         """
+        def stop_spinner():
+            if spinner is not None:
+                _output.log(mode.PROCESS_SPINNER, spinner, process_active=False)
+
+        if self.skip:
+            stop_spinner()
+            msg = f'Skipping Stage {self.sequence}{": " + self.name if self.name else ""} per user request.'
+            _output.log(mode.SKIP, msg)
+            return ExitCode.SKIPPED
+
         # Setup if not already setup.
         if not self.is_setup:
             self.setup()
@@ -442,18 +521,26 @@ class Stage:
         try:
             result = self._command_runner.provision()
         except KeyboardInterrupt:
-            if spinner is not None:
-                _output.log(mode.PROCESS_SPINNER, spinner, process_active=False)
+            stop_spinner()
             raise KeyboardInterrupt
         except Exception as err:
-            if spinner is not None:
-                _output.log(mode.PROCESS_SPINNER, spinner, process_active=False)
+            stop_spinner()
             raise SetupError(exception=err)
         if not result:
+            stop_spinner()
             raise SetupError
 
         # Call the command runner's prepare function.
-        self._command_runner.prepare()
+        try:
+            self._command_runner.prepare()
+        except OSEnvironmentMismatch:
+            stop_spinner()
+            msg = (
+                f'Skipping Stage {self.sequence}{": " + self.name if self.name else ""} '
+                f'because OS is not {self._command_runner.environment.lower()}.'
+            )
+            _output.log(mode.SKIP, msg)
+            return ExitCode.SKIPPED
 
         for mac in self._macros:
             directive = self._directives[mac.sequence - 1]
@@ -468,31 +555,30 @@ class Stage:
 
             # Run the macro.
             try:
-                if spinner is not None:
-                    _output.log(mode.PROCESS_SPINNER, spinner, process_active=False)
+                stop_spinner()
                 _output.log(
                     mode.MACRO_START,
                     directive=directive,
-                    command=mac.command,
+                    command=mac.label if mac.label else mac.command,
                     sequence=mac.sequence,
                     total=len(self._macros),
                 )
                 status = self._command_runner.execute(mac)
             except Exception as err:
-                if spinner is not None:
-                    _output.log(mode.PROCESS_SPINNER, spinner, process_active=False)
+                stop_spinner()
                 raise ExecutionError(exception=err)
 
             # Handle the result.
             _output.log(
                 mode.MACRO_STATUS,
                 directive=directive,
-                command=mac.command,
+                command=mac.label if mac.label else mac.command,
                 status_code=status.exit_code,
                 sequence=mac.sequence,
                 total=len(self._macros),
             )
             self._results.append(status)
+
             if verbose:
                 if status.stdout:
                     _output.print_output(status.stdout)
@@ -508,6 +594,6 @@ class Stage:
         # Set the exit code.
         fails = map(lambda r: True if r.exit_code > 0 else False, self._results)
         if any(fails):
-            self._result = 1
+            self._result = ExitCode.FAILED
 
         return self._result
