@@ -7,9 +7,7 @@ import sys
 from colorama import Cursor, Fore, init, Style
 
 from build_magic import __version__ as version
-from build_magic.reference import OutputMethod
-
-from yaspin import yaspin
+from build_magic.reference import OutputMethod, ExitCode
 
 
 class Output:
@@ -55,6 +53,13 @@ class Output:
         """Communicates a general information message."""
         raise NotImplementedError
 
+    def skip(self, *args, **kwargs):
+        """Communicates a skipping message."""
+
+    def working_directory(self, *args, **kwargs):
+        """Communicates the current working directory."""
+        raise NotImplementedError
+
     @staticmethod
     def _display(line, err=False):
         """Prints a message to stdout.
@@ -116,19 +121,21 @@ class Basic(Output):
             message = f'{message} in {delta.total_seconds():.3f}'
         self._display(message)
 
-    def start_stage(self, stage_number=1, name=None):
+    def start_stage(self, stage_number=1, name='', description=''):
         """Indicates the beginning of a stage.
 
         :param int stage_number: The stage sequence number.
-        :param str|None name: The stage name if given.
+        :param str name: The stage name if given.
+        :param str description: The stage description if given.
         :return: None
         """
         message = f'{datetime.now().isoformat()} build-magic [ INFO  ] Starting Stage {stage_number}'
         if name:
             message = f'{message}: {name}'
+        if description:
+            message = f'{message} - {description}'
         self._display(message)
 
-
     def end_stage(self, stage_number=1, status_code=0, name=None):
         """Indicates the end of a stage.
 
@@ -137,8 +144,11 @@ class Basic(Output):
         :param str|None name: The stage name if given.
         :return: None
         """
-        result = 'DONE'
-        if status_code > 0:
+        if status_code == ExitCode.PASSED:
+            result = 'DONE'
+        elif status_code == ExitCode.SKIPPED:
+            result = 'SKIP'
+        else:
             result = 'FAIL'
         if name:
             message = f'{datetime.now().isoformat()} build-magic [ INFO  ] Stage {stage_number}: {name} - ' \
@@ -195,6 +205,25 @@ class Basic(Output):
         message = f'{datetime.now().isoformat()} build-magic [ INFO  ] OUTPUT: {msg}'
         self._display(message)
 
+    def skip(self, msg):
+        """Communicates a skipping message.
+
+        :param str msg: The message to print.
+        :return: None
+        """
+        msg = msg.rstrip()
+        message = f'{datetime.now().isoformat()} build-magic [ SKIP  ] OUTPUT: {msg}'
+        self._display(message)
+
+    def working_directory(self, wd):
+        """Communicates the current working directory.
+
+        :param str wd: The path to display.
+        :return: None
+        """
+        message = f'{datetime.now().isoformat()} build-magic [ INFO  ] Current working directory: {wd}'
+        self._display(message)
+
     def process_spinner(self, *args, **kwargs):
         """Not used by the Basic Output class."""
         return
@@ -249,16 +278,19 @@ class Tty(Output):
             message = f'build-magic{emoji} finished at {datetime.now().strftime("%c")}'
         self._display(message)
 
-    def start_stage(self, stage_number=1, name=None):
+    def start_stage(self, stage_number=1, name='', description=''):
         """Indicates the beginning of a stage.
 
         :param int stage_number: The stage sequence number.
-        :param str|None name: The stage name if given.
+        :param str name: The stage name if given.
+        :param str description: The stage description if given.
         :return: None
         """
         message = f'Starting Stage {stage_number}'
         if name:
             message += f': {name}'
+        if description:
+            message += f' - {description}'
         self._display(message)
 
     def end_stage(self, stage_name=1, status_code=0, name=None):
@@ -269,9 +301,13 @@ class Tty(Output):
         :param str|None name: The stage name if given.
         :return: None
         """
-        color = Fore.GREEN + Style.BRIGHT
-        result = 'DONE'
-        if status_code > 0:
+        if status_code == ExitCode.PASSED:
+            color = Fore.GREEN + Style.BRIGHT
+            result = 'DONE'
+        elif status_code == ExitCode.SKIPPED:
+            color = Fore.YELLOW + Style.BRIGHT
+            result = 'SKIPPED'
+        else:
             color = Fore.RED + Style.BRIGHT
             result = 'FAILED'
         if name:
@@ -294,16 +330,18 @@ class Tty(Output):
         :return: None
         """
         width = self.get_width()
+        command = command.strip()
         status = Fore.YELLOW + Style.BRIGHT + 'RUNNING' + Style.RESET_ALL
         seq_length = len(str(total))
         seq = f'( {sequence:>{seq_length}}/{total:>{seq_length}} )'
         spacing = width - 23 - len(command) - len(seq)
-        if len(command) + 12 + len(seq) > width - 11:
-            command = command[:width - 24 - len(seq)] + '....'
         if not command:
             message = f'{directive.upper()} {"." * spacing}'
+        elif len(command) + 12 + len(seq) > width - 11:
+            command = command[:width - 27 - len(seq)] + ' ....'
+            message = f'{seq} {directive.upper():<8}: {command} {status:<8}'
         else:
-            message = f'{seq} {directive.upper():<8}: {command} {"." * spacing} {status}'
+            message = f'{seq} {directive.upper():<8}: {command} {"." * spacing} {status:<8}'
         self._display(message)
 
     def macro_status(self, directive='', command='', status_code=0, sequence=1, total=1):
@@ -349,7 +387,26 @@ class Tty(Output):
         message = 'OUTPUT: {}'.format(msg)
         self._display(message)
 
-    def process_spinner(self, spinner, process_active=False):
+    def skip(self, msg):
+        """Communicates a skipping message.
+
+        :param str msg: The message to print.
+        :return: None
+        """
+        msg = msg.rstrip()
+        self._display(Fore.YELLOW + str(msg) + Style.RESET_ALL, err=True)
+
+    def working_directory(self, wd):
+        """Communicates the current working directory.
+
+        :param str wd: The path to display.
+        :return: None
+        """
+        message = f'=> {Style.BRIGHT}Current working directory:{Style.RESET_ALL} {wd}'
+        self._display(message)
+
+    @staticmethod
+    def process_spinner(spinner, process_active=False):
         """Indicates whether a process is underway.
 
         :param yaspin  spinner: Yaspin spinner object.
@@ -401,6 +458,14 @@ class Silent(Output):
         """Communicates a general information message."""
         return
 
+    def skip(self, *args, **kwargs):
+        """Communicates a skipping message."""
+        return
+
+    def working_directory(self, *args, **kwargs):
+        """Communicates the current working directory."""
+        return
+
     def process_spinner(self, *args, **kwargs):
         """Indicates whether a process is underway."""
         return
